@@ -1,5 +1,5 @@
 /*
- * "$Id: auth.c 10776 2012-12-17 22:17:08Z mike $"
+ * "$Id: auth.c 11500 2014-01-06 22:21:15Z msweet $"
  *
  *   Authorization routines for the CUPS scheduler.
  *
@@ -513,18 +513,37 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
     socklen_t		peersize;	/* Size of peer credentials */
 #ifdef HAVE_AUTHORIZATION_H
     const char		*name;		/* Authorizing name */
+    int			no_peer = 0;	/* Don't allow peer credentials? */
+
+   /*
+    * See if we should allow peer credentials...
+    */
 
     for (name = (char *)cupsArrayFirst(con->best->names);
          name;
          name = (char *)cupsArrayNext(con->best->names))
+    {
       if (!_cups_strncasecmp(name, "@AUTHKEY(", 9) ||
           !_cups_strcasecmp(name, "@SYSTEM"))
       {
-	cupsdLogMessage(CUPSD_LOG_ERROR,
-	                "[Client %d] PeerCred authentication not allowed for "
-	                "resource.", con->http.fd);
-	return;
+       /* Normally don't want peer credentials if we need an auth key... */
+	no_peer = 1;
       }
+      else if (!_cups_strcasecmp(name, "@OWNER"))
+      {
+       /* but if @OWNER is present then we allow it... */
+        no_peer = 0;
+        break;
+      }
+    }
+
+    if (no_peer)
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR,
+		      "[Client %d] PeerCred authentication not allowed for "
+		      "resource per AUTHKEY policy.", con->http.fd);
+      return;
+    }
 #endif /* HAVE_AUTHORIZATION_H */
 
     if ((pwd = getpwnam(authorization + 9)) == NULL)
@@ -1164,7 +1183,7 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 
 
     if (sscanf(authorization, "%255s", scheme) != 1)
-      strcpy(scheme, "UNKNOWN");
+      strlcpy(scheme, "UNKNOWN", sizeof(scheme));
 
     cupsdLogMessage(CUPSD_LOG_ERROR,
                     "[Client %d] Bad authentication data \"%s ...\"",
@@ -1283,6 +1302,8 @@ cupsdCheckAuth(unsigned     ip[4],	/* I - Client address */
           netip6[3] = htonl(ip[3]);
 #endif /* AF_INET6 */
 
+	  cupsdNetIFUpdate();
+
           if (!strcmp(mask->mask.name.name, "*"))
 	  {
 #ifdef __APPLE__
@@ -1298,8 +1319,6 @@ cupsdCheckAuth(unsigned     ip[4],	/* I - Client address */
 	    * Check against all local interfaces...
 	    */
 
-            cupsdNetIFUpdate();
-
 	    for (iface = (cupsd_netif_t *)cupsArrayFirst(NetIFList);
 		 iface;
 		 iface = (cupsd_netif_t *)cupsArrayNext(NetIFList))
@@ -2588,5 +2607,5 @@ to64(char          *s,			/* O - Output string */
 
 
 /*
- * End of "$Id: auth.c 10776 2012-12-17 22:17:08Z mike $".
+ * End of "$Id: auth.c 11500 2014-01-06 22:21:15Z msweet $".
  */
