@@ -1,9 +1,9 @@
 /*
- * "$Id: conf.h 10710 2012-11-26 18:26:01Z mike $"
+ * "$Id: conf.h 11202 2013-07-26 21:31:54Z msweet $"
  *
  *   Configuration file definitions for the CUPS scheduler.
  *
- *   Copyright 2007-2012 by Apple Inc.
+ *   Copyright 2007-2013 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -20,9 +20,10 @@
 
 typedef enum
 {
-  CUPSD_LOG_PPD = -4,			/* Used internally for PPD keywords */
+  CUPSD_LOG_PPD = -5,			/* Used internally for PPD keywords */
   CUPSD_LOG_ATTR,			/* Used internally for attributes */
-  CUPSD_LOG_STATE,			/* Used internally for state-reasons */
+  CUPSD_LOG_STATE,			/* Used internally for printer-state-reasons */
+  CUPSD_LOG_JOBSTATE,			/* Used internally for job-state-reasons */
   CUPSD_LOG_PAGE,			/* Used internally for page logging */
   CUPSD_LOG_NONE,
   CUPSD_LOG_EMERG,			/* Emergency issues */
@@ -149,8 +150,6 @@ VAR char		*AccessLog		VALUE(NULL),
 					/* Temporary directory */
 			*Printcap		VALUE(NULL),
 					/* Printcap file */
-			*PrintcapGUI		VALUE(NULL),
-					/* GUI program to use for IRIX */
 			*FontPath		VALUE(NULL),
 					/* Font search path */
 			*RemoteRoot		VALUE(NULL),
@@ -173,6 +172,8 @@ VAR int			ClassifyOverride	VALUE(0),
 					/* Which errors are fatal? */
 			StrictConformance	VALUE(FALSE),
 					/* Require strict IPP conformance? */
+			SyncOnClose		VALUE(FALSE),
+					/* Call fsync() when closing files? */
 			LogFilePerm		VALUE(0644);
 					/* Permissions for log files */
 VAR cupsd_loglevel_t	LogLevel		VALUE(CUPSD_LOG_WARN);
@@ -296,5 +297,5 @@ extern int	cupsdWriteErrorLog(int level, const char *message);
 
 
 /*
- * End of "$Id: conf.h 10710 2012-11-26 18:26:01Z mike $".
+ * End of "$Id: conf.h 11202 2013-07-26 21:31:54Z msweet $".
  */
