@@ -1,10 +1,10 @@
 #!/bin/sh
 #
-# "$Id: 5.8-cancel.sh 10097 2011-11-02 05:35:38Z mike $"
+# "$Id: 5.8-cancel.sh 11396 2013-11-06 20:09:03Z msweet $"
 #
 #   Test the cancel command.
 #
-#   Copyright 2007-2011 by Apple Inc.
+#   Copyright 2007-2013 by Apple Inc.
 #   Copyright 1997-2006 by Easy Software Products, all rights reserved.
 #
 #   These coded instructions, statements, and computer programs are the
@@ -40,6 +40,18 @@ else
 fi
 echo ""
 
+echo "Purge All Test"
+echo ""
+echo "    cancel -a -x"
+$VALGRIND ../systemv/cancel -a -x 2>&1
+if test $? != 0; then
+	echo "    FAILED"
+	exit 1
+else
+	echo "    PASSED"
+fi
+echo ""
+
 #
-# End of "$Id: 5.8-cancel.sh 10097 2011-11-02 05:35:38Z mike $".
+# End of "$Id: 5.8-cancel.sh 11396 2013-11-06 20:09:03Z msweet $".
 #
